@@ -20,7 +20,8 @@ if TYPE_CHECKING:
 else:
     NoReturn = None
 
-# Newer APIs from importlib_resources should arrive to stdlib importlib.resources in Python 3.9.
+# In newer Python versions, use importlib.resources from the standard library.
+# On older versions, a compatibility package must be installed from PyPI.
 if sys.version_info < (3, 9):
     import importlib_resources
 else:
@@ -88,9 +89,9 @@ def get_supported_docformats() -> Iterator[str]:
     """
     Get the list of currently supported docformat.
     """
-    for fileName in importlib_resources.contents('pydoctor.epydoc.markup'):
+    for fileName in (path.name for path in importlib_resources.files('pydoctor.epydoc.markup').iterdir()):
         moduleName = getmodulename(fileName)
-        if moduleName is None or moduleName == '__init__':
+        if moduleName is None or moduleName.startswith("_"):
             continue
         else:
             yield moduleName
