@@ -1,18 +1,20 @@
 """
-Convert epydoc markup into renderable content.
+Convert L{pydoctor.epydoc} parsed markup into renderable content.
 """
 
 from collections import defaultdict
 from importlib import import_module
 from typing import (
-    Callable, ClassVar, DefaultDict, Dict, Iterable, Iterator, List, Mapping,
-    Optional, Sequence, Tuple, Union
+    TYPE_CHECKING, Callable, ClassVar, DefaultDict, Dict, Generator, Iterable,
+    Iterator, List, Mapping, Optional, Sequence, Tuple, Union
 )
 import ast
+import sys
 import itertools
 
 import astor
 import attr
+import docutils.nodes
 
 from pydoctor import model
 from pydoctor.epydoc.markup import Field as EpydocField, ParseError
@@ -20,14 +22,19 @@ from twisted.web.template import Tag, tags
 from pydoctor.epydoc.markup import DocstringLinker, ParsedDocstring
 import pydoctor.epydoc.markup.plaintext
 
+if TYPE_CHECKING:
+    from twisted.web.template import Flattenable
 
 def get_parser(obj: model.Documentable) -> Callable[[str, List[ParseError]], ParsedDocstring]:
-    formatname = obj.system.options.docformat
+    
+    # Use module's __docformat__ if specified, else use system's.
+    docformat = obj.module.docformat or obj.system.options.docformat
+    
     try:
-        mod = import_module('pydoctor.epydoc.markup.' + formatname)
+        mod = import_module(f'pydoctor.epydoc.markup.{docformat}')
     except ImportError as e:
         msg = 'Error trying to import %r parser:\n\n    %s: %s\n\nUsing plain text formatting only.'%(
-            formatname, e.__class__.__name__, e)
+            docformat, e.__class__.__name__, e)
         obj.system.msg('epydoc2stan', msg, thresh=-1, once=True)
         mod = pydoctor.epydoc.markup.plaintext
     return mod.parse_docstring # type: ignore[attr-defined, no-any-return]
@@ -46,7 +53,7 @@ def get_docstring(
     return None, None
 
 
-def taglink(o: model.Documentable, page_url: str, label: Optional[str] = None) -> Tag:
+def taglink(o: model.Documentable, page_url: str, label: Optional["Flattenable"] = None) -> Tag:
     if not o.isVisible:
         o.system.msg("html", "don't link to %s"%o.fullName())
 
@@ -100,7 +107,7 @@ class _EpydocLinker(DocstringLinker):
         """
         return self.obj.system.intersphinx.getLink(name)
 
-    def link_to(self, identifier: str, label: str) -> Tag:
+    def link_to(self, identifier: str, label: "Flattenable") -> Tag:
         fullID = self.obj.expandName(identifier)
 
         target = self.obj.system.objForFullName(fullID)
@@ -109,12 +116,12 @@ class _EpydocLinker(DocstringLinker):
 
         url = self.look_for_intersphinx(fullID)
         if url is not None:
-            return tags.a(label, href=url)  # type: ignore[no-any-return]
+            return tags.a(label, href=url)
 
-        return tags.transparent(label)  # type: ignore[no-any-return]
+        return tags.transparent(label)
 
-    def link_xref(self, target: str, label: str, lineno: int) -> Tag:
-        xref: Union[Tag, str]
+    def link_xref(self, target: str, label: "Flattenable", lineno: int) -> Tag:
+        xref: "Flattenable"
         try:
             resolved = self._resolve_identifier_xref(target, lineno)
         except LookupError:
@@ -200,10 +207,8 @@ class _EpydocLinker(DocstringLinker):
         # Examine every module and package in the system and see if 'identifier'
         # names an object in each one.  Again, if more than one object is
         # found, complain.
-        target = self.look_for_name(identifier, itertools.chain(
-            self.obj.system.objectsOfType(model.Module),
-            self.obj.system.objectsOfType(model.Package)),
-            lineno)
+        target = self.look_for_name(
+            identifier, self.obj.system.objectsOfType(model.Module), lineno)
         if target is not None:
             return target
 
@@ -219,50 +224,97 @@ class _EpydocLinker(DocstringLinker):
 
 @attr.s(auto_attribs=True)
 class FieldDesc:
+    """
+    Combines informations from multiple L{Field} objects into one.
+
+    Example::
+
+       :param foo: description of parameter foo
+       :type foo:  SomeClass
+
+    """
     _UNDOCUMENTED: ClassVar[Tag] = tags.span(class_='undocumented')("Undocumented")
 
     name: Optional[str] = None
     type: Optional[Tag] = None
     body: Optional[Tag] = None
 
-    def format(self) -> Iterator[Tag]:
+    def format(self) -> Generator[Tag, None, None]:
+        """
+        @return: Iterator that yields one or two C{tags.td}.
+        """
         formatted = self.body or self._UNDOCUMENTED
-        if self.type is not None:
-            formatted = tags.transparent(formatted, ' (type: ', self.type, ')')
-
-        name = self.name
-        if name is None:
-            yield tags.td(formatted, colspan="2")
+        fieldNameTd: List[Tag] = []
+        if self.name:
+            _name = tags.span(class_="fieldArg")(self.name)
+            if self.type:
+                _name(":")
+            fieldNameTd.append(_name)
+        if self.type:
+            fieldNameTd.append(self.type)
+        if fieldNameTd:
+            #  <name>: <type> | <desc>
+            yield tags.td(class_="fieldArgContainer")(*fieldNameTd)
+            yield tags.td(class_="fieldArgDesc")(formatted)
         else:
-            yield tags.td(name, class_="fieldArg")
-            yield tags.td(formatted)
-
+            #  <desc>
+            yield tags.td(formatted, colspan="2")
 
 class RaisesDesc(FieldDesc):
     """Description of an exception that can be raised by function/method."""
 
-    def format(self) -> Iterator[Tag]:
-        yield tags.td(self.type, class_="fieldArg")
+    def format(self) -> Generator[Tag, None, None]:
+        assert self.type is not None  # TODO: Why can't it be None?
+        yield tags.td(tags.code(self.type), class_="fieldArgContainer")
         yield tags.td(self.body or self._UNDOCUMENTED)
 
 
 def format_desc_list(label: str, descs: Sequence[FieldDesc]) -> Iterator[Tag]:
-    first = True
-    for d in descs:
-        if first:
-            row = tags.tr(class_="fieldStart")
-            row(tags.td(class_="fieldName")(label))
-            first = False
-        else:
-            row = tags.tr()
-            row(tags.td())
-        yield row(d.format())
+    """
+    Format list of L{FieldDesc}. Used for param, returns, raises, etc.
 
+    Generates a 2-columns layout as follow::
+
+        +------------------------------------+
+        | <label>                            |
+        | <name>: <type> |     <desc>        |
+        | <name>: <type> |     <desc>        |
+        +------------------------------------+
+
+    If the fields don't have type or name information,
+    generates the same output as L{format_field_list}::
+
+        +------------------------------------+
+        | <label>                            |
+        | <desc ... >                        |
+        +------------------------------------+
+
+    @returns: Each row as iterator or None if no C{descs} id provided.
+    """
+    if not descs:
+        return
+    # <label>
+    row = tags.tr(class_="fieldStart")
+    row(tags.td(class_="fieldName", colspan="2")(label))
+    # yield the first row.
+    yield row
+    # yield descriptions.
+    for d in descs:
+        row = tags.tr()
+        # <name>: <type> |     <desc>
+        # or
+        # <desc ... >
+        row(d.format())
+        yield row
 
 @attr.s(auto_attribs=True)
 class Field:
-    """Like pydoctor.epydoc.markup.Field, but without the gross accessor
+    """Like L{pydoctor.epydoc.markup.Field}, but without the gross accessor
     methods and with a formatted body.
+
+    Example::
+
+        @note: some other information
     """
 
     tag: str
@@ -290,20 +342,31 @@ class Field:
 
 
 def format_field_list(singular: str, plural: str, fields: Sequence[Field]) -> Iterator[Tag]:
+    """
+    Format list of L{Field} object. Used for notes, see also, authors, etc.
+
+    Generates a 2-columns layout as follow::
+
+        +------------------------------------+
+        | <label>                            |
+        | <desc ... >                        |
+        +------------------------------------+
+
+    @returns: Each row as iterator
+    """
+    if not fields:
+        return
+
     label = singular if len(fields) == 1 else plural
-    first = True
+    row = tags.tr(class_="fieldStart")
+    row(tags.td(class_="fieldName", colspan="2")(label))
+    yield row
+
     for field in fields:
-        if first:
-            row = tags.tr(class_="fieldStart")
-            row(tags.td(class_="fieldName")(label))
-            first=False
-        else:
-            row = tags.tr()
-            row(tags.td())
+        row = tags.tr()
         row(tags.td(colspan="2")(field.format()))
         yield row
 
-
 class FieldHandler:
 
     def __init__(self, obj: model.Documentable):
@@ -508,6 +571,7 @@ class FieldHandler:
         r += format_desc_list('Parameters', self.parameter_descs)
         if self.return_desc:
             r += format_desc_list('Returns', [self.return_desc])
+
         r += format_desc_list("Raises", self.raise_descs)
         for s_p_l in (('Author', 'Authors', self.authors),
                       ('See Also', 'See Also', self.seealsos),
@@ -518,9 +582,9 @@ class FieldHandler:
             r += format_desc_list(f"Unknown Field: {kind}", fieldlist)
 
         if any(r):
-            return tags.table(class_='fieldTable')(r) # type: ignore[no-any-return]
+            return tags.table(class_='fieldTable')(r)
         else:
-            return tags.transparent # type: ignore[no-any-return]
+            return tags.transparent
 
 
 def _is_none_literal(node: ast.expr) -> bool:
@@ -551,7 +615,8 @@ def parse_docstring(
         This can differ from C{obj} if the docstring is inherited.
     """
 
-    parser = get_parser(obj)
+    # the docstring should be parsed using the format of the module it was inherited from
+    parser = get_parser(source)
     errs: List[ParseError] = []
     try:
         pdoc = parser(doc, errs)
@@ -615,7 +680,8 @@ def format_docstring(obj: model.Documentable) -> Tag:
     ret(fh.format())
     return ret
 
-
+# TODO: FIX https://github.com/twisted/pydoctor/issues/86 
+# Use to_node() and compute shortened HTML from node tree with a visitor intead of using the raw source. 
 def format_summary(obj: model.Documentable) -> Tag:
     """Generate an shortened HTML representation of a docstring."""
 
@@ -645,7 +711,7 @@ def format_summary(obj: model.Documentable) -> Tag:
                 )
             ]
         if len(lines) > 3:
-            return tags.span(class_='undocumented')("No summary") # type: ignore[no-any-return]
+            return tags.span(class_='undocumented')("No summary")
         pdoc = parse_docstring(obj, ' '.join(lines), source)
 
     try:
@@ -653,38 +719,37 @@ def format_summary(obj: model.Documentable) -> Tag:
     except Exception:
         # This problem will likely be reported by the full docstring as well,
         # so don't spam the log.
-        return tags.span(class_='undocumented')("Broken description") # type: ignore[no-any-return]
+        return tags.span(class_='undocumented')("Broken description")
 
-    content = [stan] if stan.tagName else stan.children
+    content: Sequence["Flattenable"] = [stan] if stan.tagName else stan.children
     if content and isinstance(content[0], Tag) and content[0].tagName == 'p':
         content = content[0].children
-    return tags.span(*content) # type: ignore[no-any-return]
+    return Tag('')(*content)
 
 
 def format_undocumented(obj: model.Documentable) -> Tag:
     """Generate an HTML representation for an object lacking a docstring."""
 
-    subdocstrings: DefaultDict[str, int] = defaultdict(int)
-    subcounts: DefaultDict[str, int]  = defaultdict(int)
-    for subob in obj.contents.values():
-        k = subob.kind.lower()
-        subcounts[k] += 1
-        if subob.docstring is not None:
-            subdocstrings[k] += 1
-    if isinstance(obj, model.Package):
-        subcounts['module'] -= 1
+    sub_objects_with_docstring_count: DefaultDict[model.DocumentableKind, int] = defaultdict(int)
+    sub_objects_total_count: DefaultDict[model.DocumentableKind, int]  = defaultdict(int)
+    for sub_ob in obj.contents.values():
+        k = sub_ob.kind
+        sub_objects_total_count[k] += 1
+        if sub_ob.docstring is not None:
+            sub_objects_with_docstring_count[k] += 1
 
     tag: Tag = tags.span(class_='undocumented')
-    if subdocstrings:
-        plurals = {'class': 'classes'}
+    if sub_objects_with_docstring_count:
+        
         kind = obj.kind
         assert kind is not None # if kind is None, object is invisible
         tag(
-            "No ", kind.lower(), " docstring; ",
+            "No ", format_kind(kind).lower(), " docstring; ",
             ', '.join(
-                f"{subdocstrings[k]}/{subcounts[k]} "
-                f"{plurals.get(k, k + 's')}"
-                for k in sorted(subcounts)
+                f"{sub_objects_with_docstring_count[k]}/{sub_objects_total_count[k]} "
+                f"{format_kind(k, plural=sub_objects_with_docstring_count[k]>=2).lower()}"
+                
+                for k in sorted(sub_objects_total_count, key=(lambda x:x.value))
                 ),
             " documented"
             )
@@ -718,10 +783,14 @@ class AnnotationDocstring(ParsedDocstring):
         ParsedDocstring.__init__(self, ())
         self.annotation = annotation
 
+    def has_body(self) -> bool:
+        return True
+
     def to_stan(self, docstring_linker: DocstringLinker) -> Tag:
-        tag: Tag = tags.code
-        tag(_AnnotationFormatter(docstring_linker).visit(self.annotation))
-        return tag
+        return tags.code(_AnnotationFormatter(docstring_linker).visit(self.annotation))
+    
+    def to_node(self) -> docutils.nodes.document:
+        raise NotImplementedError()
 
 
 class _AnnotationFormatter(ast.NodeVisitor):
@@ -743,13 +812,16 @@ class _AnnotationFormatter(ast.NodeVisitor):
         else:
             return self.generic_visit(node)
 
-    def _handle_sequence(self, tag: Tag, sequence: Iterable[ast.expr]) -> None:
+    def _handle_sequence(self, tag: Tag, sequence: List[ast.expr]) -> None:
         first = True
-        for elem in sequence:
+        for i, elem in enumerate(sequence):
             if first:
                 first = False
             else:
+                # Add an potential line break for long types, not when it's the last bracket, though.
                 tag(', ')
+                if i < len(sequence)-1:
+                    tag(tags.wbr)
             tag(self.visit(elem))
 
     def visit_Name(self, node: ast.Name) -> Tag:
@@ -777,9 +849,9 @@ class _AnnotationFormatter(ast.NodeVisitor):
     def visit_Subscript(self, node: ast.Subscript) -> Tag:
         tag: Tag = tags.transparent
         tag(self.visit(node.value))
-        tag('[')
+        tag('[', tags.wbr)
         sub: ast.AST = node.slice
-        if isinstance(sub, ast.Index):
+        if sys.version_info < (3, 9) and isinstance(sub, ast.Index):
             # In Python < 3.9, non-slices are always wrapped in an Index node.
             sub = sub.value
         if isinstance(sub, ast.Tuple):
@@ -791,7 +863,7 @@ class _AnnotationFormatter(ast.NodeVisitor):
 
     def visit_List(self, node: ast.List) -> Tag:
         tag: Tag = tags.transparent
-        tag('[')
+        tag('[', tags.wbr)
         self._handle_sequence(tag, node.elts)
         tag(']')
         return tag
@@ -802,10 +874,10 @@ class _AnnotationFormatter(ast.NodeVisitor):
         return ret
 
 
-field_name_to_human_name = {
-    'ivar': 'Instance Variable',
-    'cvar': 'Class Variable',
-    'var': 'Variable',
+field_name_to_kind = {
+    'ivar': model.DocumentableKind.INSTANCE_VARIABLE,
+    'cvar': model.DocumentableKind.CLASS_VARIABLE,
+    'var': model.DocumentableKind.VARIABLE,
     }
 
 
@@ -839,4 +911,33 @@ def extract_fields(obj: model.Documentable) -> None:
                 attrobj.parsed_type = field.body()
             else:
                 attrobj.parsed_docstring = field.body()
-                attrobj.kind = field_name_to_human_name[tag]
+                attrobj.kind = field_name_to_kind[tag]
+
+def format_kind(kind: model.DocumentableKind, plural: bool = False) -> str:
+    """
+    Transform a `model.DocumentableKind` Enum value to string. 
+    """
+    names = {
+        model.DocumentableKind.PACKAGE         : 'Package',
+        model.DocumentableKind.MODULE          : 'Module',
+        model.DocumentableKind.INTERFACE       : 'Interface',
+        model.DocumentableKind.CLASS           : 'Class',
+        model.DocumentableKind.CLASS_METHOD    : 'Class Method',
+        model.DocumentableKind.STATIC_METHOD   : 'Static Method',
+        model.DocumentableKind.METHOD          : 'Method',
+        model.DocumentableKind.FUNCTION        : 'Function',
+        model.DocumentableKind.CLASS_VARIABLE  : 'Class Variable',
+        model.DocumentableKind.ATTRIBUTE       : 'Attribute',
+        model.DocumentableKind.INSTANCE_VARIABLE : 'Instance Variable',
+        model.DocumentableKind.PROPERTY        : 'Property',
+        model.DocumentableKind.VARIABLE        : 'Variable',
+        model.DocumentableKind.SCHEMA_FIELD    : 'Attribute',
+    }
+    plurals = {
+        model.DocumentableKind.CLASS           : 'Classes', 
+        model.DocumentableKind.PROPERTY        : 'Properties',
+    }
+    if plural:
+        return plurals.get(kind, names[kind] + 's')
+    else:
+        return names[kind]
